/*! 
 * \file verilogscanner.h
 * \author Jonas Norling, Johan Lissing
 * \date 2005-11-27
 * \version 1.0 Added comments and this header.
 *
 * This is the interface to the Verilog parser.
 */

/*
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby 
 * granted. No representations are made about the suitability of this software 
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 * Documents produced by Doxygen are derivative works derived from the
 * input used in their production; they are not affected by this license.
 *
 */

#ifndef VERILOGSCANNER_H
#define VERILOGSCANNER_H

#include "parserintf.h"

/**
 * \brief Verilog parser
 *
 * This is a scanner/parser for Verilog (pre-2001, some 2001 features) that
 * plugs into Doxygen's language parser interface.
 */
class VerilogLanguageScanner : public ParserInterface
{
 public:
  /** Parses a single input file with the goal to build an Entry tree. 
   *  @param[in] fileName    The full name of the file.
   *  @param[in] fileBuf     The contents of the file (zero terminated).
   *  @param[in,out] root    The root of the tree of Entry *nodes 
   *             representing the information extracted from the file.
   */
  void parseInput(const char *fileName, 
		  const char *fileBuf, 
		  Entry *root);

  /** Returns TRUE if the language identified by \a extension needs
   *  the C preprocessor to be run before feed the result to the input
   *  parser.
   *  @see parseInput()
   */
  bool needsPreprocessing(const QCString &extension);
    
  /** Parses a source file or fragment with the goal to produce
   *  highlighted and cross-referenced output.
   *  THIS METHOD IS NOT IMPLEMENTED IN THE VERILOG PARSER
   */
  void parseCode(CodeOutputInterface &codeOutIntf,
		 const char *scopeName,
		 const QCString &input,
		 bool isExampleBlock,
		 const char *exampleName=0,
		 FileDef *fileDef=0,
		 int startLine=-1,
		 int endLine=-1,
		 bool inlineFragment=FALSE,
		 MemberDef *memberDef=0);

  /** Resets the state of the code parser.
   *  @see parseCode()
   *  THIS METHOD IS NOT IMPLEMENTED IN THE VERILOG PARSER
   */
  virtual void resetCodeParserState();

  /** Callback function called by the comment block scanner.
   *  THIS METHOD IS NOT IMPLEMENTED IN THE VERILOG PARSER
   */
  virtual void parsePrototype(const char *text);
};


#endif
