#include <doxygen.h>
#include <scanner.h>
#include <verilogscanner.h>
#include <bufstr.h>
#include <commentcnv.h>
#include <message.h>
#include <config.h>
#include "treeprinter.h"

#include <qfile.h>
#include <qfileinfo.h>

#include <iostream>

using namespace std;

//----------------------------------------------------------------------------
// reads a file into an array and filters out any 0x00 and 0x06 bytes,
// because these are special for the parser.
static void copyAndFilterFile(const char *fileName,BufStr &dest)
{
  // try to open file
  int size=0;
  uint oldPos = dest.curPos();
  //printf(".......oldPos=%d\n",oldPos);

  QFileInfo fi(fileName);
  if (!fi.exists()) return;
  QCString filterName = getFileFilter(fileName);
  if (filterName.isEmpty())
  {
    QFile f(fileName);
    if (!f.open(IO_ReadOnly))
    {
      err("Error: could not open file %s\n",fileName);
      return;
    }
    size=fi.size();
    // read the file
    dest.skip(size);
    if (f.readBlock(dest.data()+oldPos,size)!=size)
    {
      err("Error while reading file %s\n",fileName);
      return;
    }
  }
  else
  {
    QCString cmd=filterName+" \""+fileName+"\"";
    FILE *f=popen(cmd,"r");
    if (!f)
    {
      err("Error: could not execute filter %s\n",filterName.data());
      return;
    }
    const int bufSize=1024;
    char buf[bufSize];
    int numRead;
    while ((numRead=fread(buf,1,bufSize,f))>0) 
    {
      //printf(">>>>>>>>Reading %d bytes\n",numRead);
      dest.addArray(buf,numRead),size+=numRead;
    }
    pclose(f);
  }
  // filter unwanted bytes from the resulting data
  uchar conv[256];
  int i;
  for (i=0;i<256;i++) conv[i]=i;
  conv[0x06]=0x20; // replace the offending characters with spaces
  conv[0x00]=0x20;
  // remove any special markers from the input
  uchar *p=(uchar *)dest.data()+oldPos;
  for (i=0;i<size;i++,p++) *p=conv[*p];
  // and translate CR's
  int newSize=filterCRLF(dest.data()+oldPos,size);
  //printf("filter char at %p size=%d newSize=%d\n",dest.data()+oldPos,size,newSize);
  if (newSize!=size) // we removed chars
  {
    dest.shrink(oldPos+newSize); // resize the array
    //printf(".......resizing from %d to %d result=[%s]\n",oldPos+size,oldPos+newSize,dest.data());
  }
}

int main(int argc, char *argv[])
{
  if (argc < 2) {
    cerr << "Usage: " << argv[0] << " <filename>*" << endl;
    return 1;
  }

  initDoxygen();
  Doxygen::inputNameDict = new FileNameDict(10007);
  readConfiguration(0, 0);

  for (int i = 1; i < argc; i++) {
    ParserInterface *parser;
    Entry *root = new Entry; /* Leaky! */
    QCString fileName = argv[i];
    QCString extension;
    int ei = fileName.findRev('.');
    if (ei!=-1) extension=fileName.right(fileName.length()-ei);

    QFileInfo fi(fileName);
    BufStr preBuf(fi.size()+4096);
    BufStr *bufPtr = &preBuf;

    cout << "Reading " << fileName << "..." << endl;

    copyAndFilterFile(fileName, *bufPtr);
    bufPtr->addChar('\n');
    BufStr convBuf(bufPtr->curPos()+1024);
    convertCppComments(&preBuf, &convBuf, fileName);
    convBuf.addChar('\0');

    //# I prefer it this way, although im not religious about it
    if (extension == ".cpp" || extension == ".h")
    {
       parser = new CLanguageScanner;
    }
    else if (extension == ".v")
    {
       parser = new VerilogLanguageScanner;
    }
    else
    {
      cerr << "No parser for this language" << endl;
      continue;
    }

    parser->parseInput(fileName, convBuf.data(), root);
    delete parser;

    TreePrinter tp(true, 5);
    tp.print(cout, root);
  }
  
  return 0;
}
