/*! 
 * \file treeprinter.h
 * \author Thobias Bergkvist
 * \date 2005-11-15
 * \version 0.1 File created
 * \version 0.2 Added extended printing implementation.
 * \version 0.3 see treeprinter.cpp.
 *
 * A TreePrinter is used for printing a tree of Entry nodes.
 */

#ifndef TREEPRINTER_H
#define TREEPRINTER_H

#include "entry.h"
#include <string>
#include <iostream>

/*!
 * The class TreePrinter is used for printing a tree of Entry nodes.
 */
class TreePrinter
{

public:

  /*!
   * The constructor.
   * \param extended Print extended information.
   * \param indent Indent value.
   */
  TreePrinter(bool extended = false, int indent = 3);

  /*!
   * A public function used for printing a tree of Entry nodes.
   * \param os An outputstream used to print the tree to.
   * \param node The root of the tree to be printed.
   * \return nothing.
   */
  void print(std::ostream& os, const Entry* node);

 private:

  /*!
   * A private function used for printing out a tree of Entry nodes.
   * This function is called by the public print function.
   * \param os An outputstream used to print the tree to.
   * \param node An entry to be printed.
   * \param indent The indent value.
   * \return nothing.
   */
  void print(std::ostream& os, const Entry* node, int indent);

  /*!
   * Translates an entry section to readable form.
   * \param section The section number.
   * \return The section name.
   */
  std::string section(int section);

  //! extended information is printed if this variable is true.
  bool ext;

  //! the number of indent steps.
  int tab;
};

#endif // TREEPRINTER_H

// EOF
